library ieee;
use ieee.std_logic_1164.all;

package ADV212_command is 

-- Direct Register Adress---- Register SWFLAG  R, rest R/W
constant PIXEL   : std_logic_vector (3 downto 0) := x"0";  --- Pixel FIFO Access Register
constant CODE    : std_logic_vector (3 downto 0) := x"1";  --- Compressed Code-Stream Access Register
constant ATTR    : std_logic_vector (3 downto 0) := x"2";  --- ATTR FIFO AccessRegister
-- x"03" und x"04" reserviert
constant EIRQIE  : std_logic_vector (3 downto 0) := x"5";  --- External Interrupt Enabled
constant EIRQFLG : std_logic_vector (3 downto 0) := x"6";  --- External interrupt Flags
constant SWFLAG  : std_logic_vector (3 downto 0) := x"7";  ---  Software Flag Register
constant BMODE   : std_logic_vector (3 downto 0) := x"8";  --- Bus Mode Configuration Register
constant MMODE   : std_logic_vector (3 downto 0) := x"9";  --- Miscellaneous Mode Register
constant STAGE   : std_logic_vector (3 downto 0) := x"A";  --- Staging Register
constant IADDR   : std_logic_vector (3 downto 0) := x"B";  --- Indirect Adress Register
constant IDATA   : std_logic_vector (3 downto 0) := x"C";  --- Indirect Data Register
constant BOOT    : std_logic_vector (3 downto 0) := x"D";  --- Boot Mode Register
constant PLL_HI  : std_logic_vector (3 downto 0) := x"E";  --- PLL Control Register - High Byte
constant PLL_LO  : std_logic_vector (3 downto 0) := x"F";  --- PLL Control Register - Low Byte

-- Internal Hardware Register--

constant PMODE1   		 : std_logic_vector (31 downto 0) := x"FFFF0400";  -- Pixel/Video Format--R/W
constant COMP_CNT_STATUS : std_logic_vector (31 downto 0) := x"FFFF0404";  -- Component count--R
constant LINE_CNT_STATUS : std_logic_vector (31 downto 0) := x"FFFF0408";  -- Line count--R
constant XTOT   		 : std_logic_vector (31 downto 0) := x"FFFF040C";  -- Total Samples per Line--R/W
constant YTOT   		 : std_logic_vector (31 downto 0) := x"FFFF0410";  -- Total Lines per Frame--R/W
constant F0_START   	 : std_logic_vector (31 downto 0) := x"FFFF0414";  -- Start Line of Field 0- R/W
constant F1_START   	 : std_logic_vector (31 downto 0) := x"FFFF0418";  -- Start Line of Field 1--R/W
constant V0_START   	 : std_logic_vector (31 downto 0) := x"FFFF041C";  -- Start of active Video Field 0--R/W
constant V1_START   	 : std_logic_vector (31 downto 0) := x"FFFF0420";  -- Start of active Video Field 1--R/W
constant V0_END   	 	 : std_logic_vector (31 downto 0) := x"FFFF0424";  -- End of active Video Field 0--R/W
constant V1_END1   		 : std_logic_vector (31 downto 0) := x"FFFF0428";  -- End of active Video Field 1--R/W
constant PIXEL_START   	 : std_logic_vector (31 downto 0) := x"FFFF042C";  -- Horizontal start of active video--R/W
constant PIXEL_END   	 : std_logic_vector (31 downto 0) := x"FFFF0430";  -- Horizontal end of active video--R/W
constant MS_CNT_DEL   	 : std_logic_vector (31 downto 0) := x"FFFF0440";  -- Master/Slave delay--R
constant PMODE2   		 : std_logic_vector (31 downto 0) := x"FFFF0448";  -- Pixel Mode 2--R/W
constant VMODE   		 : std_logic_vector (31 downto 0) := x"FFFF044C";  -- Video Mode--R/W
constant V0_REGION_ST    : std_logic_vector (31 downto 0) := x"FFFF0450";  -- Start of vertical cropped region Field 0--R/W
constant V1_REGION_ST    : std_logic_vector (31 downto 0) := x"FFFF0454";  -- Start of vertical cropped region Field 1--R/W
constant V0_REGION_END   : std_logic_vector (31 downto 0) := x"FFFF0458";  -- End of vertical cropped region Field 0--R/W
constant V1_REGION_END   : std_logic_vector (31 downto 0) := x"FFFF045C";  -- End of vertical cropped region Field 1--R/W
constant PIXEL_START_REF : std_logic_vector (31 downto 0) := x"FFFF0460";  -- Pixel start reference--R/W
constant PIXEL_END_REF   : std_logic_vector (31 downto 0) := x"FFFF0464";  -- Pixel end reference--R/W
constant EDMOD0   		 : std_logic_vector (31 downto 0) := x"FFFF1408";  -- External DMA Mode register 0--R/W
constant EDMOD1   		 : std_logic_vector (31 downto 0) := x"FFFF1414";  -- External DMA Mode register 1--R/W
constant FFTHRP   		 : std_logic_vector (31 downto 0) := x"FFFF1418";  -- FIFO Thresold for PIXEL FIFO--R/W
constant FFTHRC   		 : std_logic_vector (31 downto 0) := x"FFFF141C";  -- FIFO Thresold for CODE FIFO--R/W
constant FFTHRA   		 : std_logic_vector (31 downto 0) := x"FFFF1420";  -- FIFO Thresold for ATTRL FIFO--R/W
constant HWREV   		 : std_logic_vector (31 downto 0) := x"FFFF14F4";  -- Hardware Revision Register


end;