library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use work.ADV212_Command.all;
use IEEE.std_logic_arith.all;


entity ADV212_ansteuern is
	port(
			-- global Control Signals
			clk:					in		std_logic;									-- Clock
			rstn:					in		std_logic:= '0';
			ADDR:					out   std_logic_vector(3 downto 0);
			HDATA:				inout	std_logic_vector(15 downto 0);
			IRQ:					in		std_logic;
			VALID:				in		std_logic;
			HOLD:					inout 	std_logic:= '0';
			ACK:					in		std_logic;
			RD,WE,CS:			out	std_logic:= '0'
			-- Command Control Signals
			
		);
end ADV212_ansteuern;

architecture ADV212_ansteuern_arch of ADV212_ansteuern is

	type STATE_TYPE IS ( idle,		
								s_PLL_HI,
								s_PLL_LO,
								
								s_wait,
								
								s_BOOT0,
							
								s_BMODE0,
								s_MMODE0,
								
								s_STAGE0,
								s_IADDR0,
								s_IDATA0,
								
								s_BOOT1,
								
								s_BMODE1,
								s_MMODE1,
								
								s_STAGE1,
								s_IADDR1,
								s_IDATA1,
								s_IADDR2,
								s_EDMOD0,
								s_IADDR3,
								s_EDMOD1,
								
								s_EIRQIE0,
								s_EIRQFLG0,
								s_EIRQFLG0_lesen,
								wait0_s,
								s_IADDR4,
								s_EDMOD2,
								
								s_IADDR5,
								s_EDMOD3,
								
								s_EIRQIE1,
								
	--							test,
								s_EIRQFLG1,
								s_EIRQFLG1_lesen,
								
	--							test1,
								s_CHECK,
								s_CHECK_lesen,
								wait_s,
								s_EIRQFLG2,
								
								s_BMODE2,
								s_MMODE2,
								
								s_read_CODE
						);

	SIGNAL curr_state : 	STATE_TYPE;
	SIGNAL next_state : 	STATE_TYPE;
	SIGNAL counter		:	std_logic:= '0';
	SIGNAL start		:  std_logic:= '0';

   ----------------------------------------------------------------------------
begin

Init: process(clk)
	begin
		if (rising_edge(clk)) then
			if (rstn = '1') then											-- Reset??
				curr_state 		<= idle;									-- Statemachine go idle
			else
				curr_state		<= next_state;							-- Process next State
			end if;
		end if;
	end process Init;
   ----------------------------------------------------------------------------	
zaehler: process(clk,start)
variable i: integer range 0 to 5 :=0;
	begin
	if CLK='1' and CLK'event then
		if start = '1' then
			if i < 5 then i:= i+1; counter <= '0';----i muss fr 27 MHz hher zhlen!!!!
			else i :=0;
			counter <= '1';
			end if;
		end if;	
	end if;		
	
	end process;	
	----------------------------------------------------------------------------	
	
	SM_encode: process (IRQ, HDATA, curr_state,VALID, HOLD,counter)
-- 
	
	begin
		case curr_state is
--------------------------------------------------------------------------------------------------------------------------------
------						Set internal Clocks																										  ------
--------------------------------------------------------------------------------------------------------------------------------
			when idle		=> WE <= '0';
									CS <= '0';
									RD <= '0';
									--ADDR <="ZZZZ";
--									HDATA <= "ZZZZZZZZZZZZZZZZ";
									start <= '0';
									next_state <= s_PLL_HI;
			when s_PLL_HI	=>	ADDR <= PLL_HI; 
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"0008";
									next_state		<= s_PLL_LO;	-- x"0008" = b1000 set WE = 1
									start <= '0';
										
			when s_PLL_LO	=>	ADDR <= PLL_LO;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';				
									HDATA <= x"0004";	-- x"0004" = b100 set WE = 1
									next_state		<= s_wait;
									start <= '1';
															
--------------------------------------------------------------------------------------------------------------------------------
------						Wait for PLL lock																											  ------
--------------------------------------------------------------------------------------------------------------------------------
				when s_wait	=> CS <= '1'; 
									WE <= '1';
									RD <= '0';
									--ADDR <="ZZZZ";
									HDATA <= "ZZZZZZZZZZZZZZZZ";			
									if counter = '1' then		-- Wait loop to PLL lock
										next_state		<= s_BOOT0;
										start <= '0';		
									else	
										next_state		<= s_wait;
										start <= '1';
									end if;
									
-------------------------------------------------------------------------------------------------------------------------------
			when s_BOOT0	=>ADDR <= BOOT;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"008A"; 				-- x"008A" = b10001010 set WE = 1
									next_state		<= s_BMODE0;
									start <= '0';
			when s_BMODE0	=>  ADDR <= BMODE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"0005";					-- x"0005" = b101 set WE = 1
									next_state		<= s_MMODE0;
									start <= '0';
			when s_MMODE0	=>  ADDR <= MMODE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"0005";					-- x"0005" = b101 set WE = 1
									next_state		<= s_STAGE0;
									start <= '0';
--------------------------------------------------------------------------------------------------------------------------------
------						Load Firmware																												  ------
--------------------------------------------------------------------------------------------------------------------------------
			when s_STAGE0	=>  ADDR <= STAGE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';				
									HDATA <= x"0005";					-- x"0005" = b101 set WE = 1
									next_state		<= s_IADDR0;
									start <= '0';		
			when s_IADDR0	=>  ADDR <= IADDR;	
									CS <= '1'; 
									WE <= '1';
									RD <= '0';			
									HDATA <= x"0000";					-- x"0000" = b0 set WE = 1
									next_state		<= s_IDATA0;
									start <= '0';
			when s_IDATA0	=> --------------------------Firmware load?????????????????????????????????????????
									CS <= '1'; 
									WE <= '1';
									RD <= '0';	
									ADDR <="ZZZZ";
									HDATA <= "ZZZZZZZZZZZZZZZZ";		
									next_state		<= s_BOOT1;
									start <= '0';
-----------------------------------------------------------------------------------------------------------------------------								
			when s_BOOT1	=>	ADDR <= BOOT;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"008D";					-- x"008D" = b10001101 set WE = 1
									next_state		<= s_BMODE1;
									start <= '0';
			when s_BMODE1	=> 	ADDR <= BMODE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';			
									HDATA <= x"0015";					-- x"0015" = b10101 set WE = 1
									next_state		<= s_MMODE1;
									start <= '0';
			when s_MMODE1	=> 	ADDR <= MMODE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"0005";					-- x"0005" = b101 set WE = 1
									next_state		<= s_STAGE1;
									start <= '0';
------------------------------------------------------------------------------------------------------------------------------
----						Set Encode Parameters																									  ------
------------------------------------------------------------------------------------------------------------------------------
			when s_STAGE1	=> 	ADDR <= STAGE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';				
									HDATA <= x"0005";					-- x"0005" = b101 set WE = 1
									next_state		<= s_IADDR1;
									start <= '0';
			when s_IADDR1	=> 	ADDR <= IADDR;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"0005";					-- x"0005" = b101 set WE = 1
									next_state		<= s_IDATA1;
									start <= '0';
			when s_IDATA1	=> ---------------------------Parameter bergeben--?????????????????????????????????????????
									CS <= '1'; 
									WE <= '1';
									RD <= '0';	
									ADDR <="ZZZZ";		
									HDATA <= "ZZZZZZZZZZZZZZZZ";
									next_state		<= s_IADDR2;
									start <= '0';									
			when s_IADDR2	=>  ADDR <= IADDR;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';			
									HDATA <= EDMOD0(15 downto 0);
									next_state		<= s_EDMOD0;
									start <= '0';
			when s_EDMOD0	=> ADDR <= IDATA;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"061A";
									next_state		<= s_IADDR3;
									start <= '0';
			when s_IADDR3	=>  ADDR <= IADDR;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= EDMOD1(15 downto 0);
									next_state		<= s_EDMOD1;
									start <= '0';
			when s_EDMOD1	=> ADDR <= IDATA;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"061A";
									next_state		<= s_EIRQIE0;
									start <= '0';
			when s_EIRQIE0	=>  ADDR <= EIRQIE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= PMODE1(15 downto 0);
									next_state		<= s_EIRQFLG0;
									start <= '0';
			when s_EIRQFLG0  =>	ADDR <= EIRQFLG;
									CS <= '1'; 
									WE <= '0';
									RD <= '1';
									--ADDR <="ZZZZ";
									HDATA <= "ZZZZZZZZZZZZZZZZ";	
									next_state		<= s_EIRQFLG0_lesen;
									start <= '0';
			when s_EIRQFLG0_lesen  =>	ADDR <= EIRQFLG;
									CS <= '1'; 
									WE <= '0';
									RD <= '1';
									ADDR <="ZZZZ";
									HDATA <= "ZZZZZZZZZZZZZZZZ";	
										if (HDATA(10) = '1' or IRQ = '0') then -- or HDATA(10) = '1') then
											next_state		<= wait0_s;
										else
											next_state		<= s_EIRQFLG0;
										end if;
									start <= '0';
			when wait0_s			=>	ADDR <= EIRQFLG;
										CS <= '1'; 
									WE <= '0';
									RD <= '1';
									HDATA <= "ZZZZZZZZZZZZZZZZ";	
									next_state		<= s_IADDR4;
			when s_IADDR4	=>  ADDR <= IADDR;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= EDMOD0(15 downto 0);
									next_state		<= s_EDMOD2;
									start <= '0';
			when s_EDMOD2	=> ADDR <= IDATA;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"061B";
									next_state		<= s_IADDR5;
									start <= '0';
			when s_IADDR5	=>  ADDR <= IADDR;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= EDMOD1(15 downto 0);
									next_state		<= s_EDMOD3;
									start <= '0';
			when s_EDMOD3	=> ADDR <= IDATA;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"061B";
									next_state		<= s_EIRQIE1;
									start <= '0';
			when s_EIRQIE1	=>  ADDR <= EIRQIE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= PMODE1(15 downto 0);
									next_state		<= s_EIRQFLG1;
									start <= '0';
--			when test		=>	ADDR <= EIRQFLG;
--									HDATA(10) <= '1'; --zum test ob es in nchsten Zustand springt
--									CS <= '1'; 
--									WE <= '1';
--									RD <= '0';
--									start <= '0';
--									next_state		<= s_EIRQFLG1;
			when s_EIRQFLG1	=>	ADDR <= EIRQFLG;
									CS <= '1'; 
									WE <= '0';
									RD <= '1';
									--ADDR <="ZZZZ";									
									HDATA <= "ZZZZZZZZZZZZZZZZ";
									next_state <=s_EIRQFLG1_lesen;					
									start <= '0';
			when s_EIRQFLG1_lesen	=>	ADDR <= EIRQFLG;
									CS <= '1'; 
									WE <= '0';
									RD <= '1';
									--ADDR <="ZZZZ";									
									HDATA <= "ZZZZZZZZZZZZZZZZ";		
									if (IRQ = '0' and HDATA(10) = '1') then
										next_state		<= s_CHECK;
									else
										next_state		<= s_EIRQFLG1_lesen;
									end if;
									start <= '0';
------------------------------------------------------------------------------------------------------------------------------
----						Check Firmware																																  ------
------------------------------------------------------------------------------------------------------------------------------
--			when test1		=>	ADDR <= SWFLAG;
--									HDATA <= x"FF82"; --zum test ob es in nchsten Zustand springt
--									CS <= '1'; 
--									WE <= '1';
--									RD <= '0';
--									start <= '0';
--									next_state		<= s_CHECK;

			when s_CHECK	=>	ADDR <= SWFLAG;
								  --HDATA <= x"FF82"; --zum test ob es in nchsten Zustand springt
									CS <= '1'; 
									WE <= '0';
									RD <= '1';
									HDATA <= "ZZZZZZZZZZZZZZZZ";		
									next_state		<= s_CHECK_lesen;
									start <= '0';
			when s_CHECK_lesen	=>	ADDR <= SWFLAG;
								  --HDATA <= x"FF82"; --zum test ob es in nchsten Zustand springt
									CS <= '1'; 
									WE <= '0';
									RD <= '1';
									HDATA <= "ZZZZZZZZZZZZZZZZ";		
									if (HDATA = x"FF82") then
										next_state		<= wait_s;
									else
										next_state		<= s_CHECK_lesen;
									end if;
									start <= '0';
			when wait_s			=>	ADDR <= SWFLAG;
										CS <= '1'; 
									WE <= '0';
									RD <= '1';
									HDATA <= "ZZZZZZZZZZZZZZZZ";	
									next_state		<= s_EIRQFLG2;
			when s_EIRQFLG2	=>  ADDR  <= EIRQFLG;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"FFFF";
									next_state <= s_BMODE2;
									start <= '0';
			when s_BMODE2	=> 	ADDR <= BMODE;
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									HDATA <= x"0015";
									next_state <= s_MMODE2;
									start <= '0';
			when s_MMODE2	=> 	ADDR <= MMODE;
									HDATA <= x"0005";
									CS <= '1'; 
									WE <= '1';
									RD <= '0';		
									if VALID ='1' then
										HOLD <= '0';
										next_state		<= s_read_CODE;
									else
										next_state		<= s_MMODE2;
									end if;
									next_state		<= s_read_CODE;
									start <= '0';
			when s_read_CODE =>
									CS <= '1'; 
									WE <= '0';
									RD <= '1';
									ADDR <="ZZZZ";
									HDATA <= "ZZZZZZZZZZZZZZZZ";	
									if (VALID = '1') and (HOLD = '0') then
											next_state		<= s_read_CODE;
										else
											next_state		<= s_MMODE2;
									end if;
									start <= '0';
		end case;
	end process SM_encode;
   ----------------------------------------------------------------------------
	
end ADV212_ansteuern_arch;

